// Copyright (c)1995 Ray Dream, Inc. All Rights Reserved.
/* $Id: ShdrFac.cpp 1.1 1996/07/19 00:15:20 Damien Exp $ */
// Class factories

#ifndef __SHDRFAC__
#include "ShdrFac.h"
#endif 

#ifndef __RAINBOW__
#include "Rainbow.h"
#endif 

#ifndef __CHECKER__
#include "Checker.h"
#endif 


// ***** ClassFactory *****
  
CheckerClassFactory::CheckerClassFactory() {
  m_cRef=0L;  // just created so no reference used
  return;
  }


CheckerClassFactory::~CheckerClassFactory() {
  return;
  }

// IUnknown methods of CheckerClassFactory                                                
STDMETHODIMP CheckerClassFactory::QueryInterface(REFIID riid, LPVOID FAR* ppv) {
  *ppv=NULL;

  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
  	*ppv=(LPVOID)this;

  if (*ppv!=NULL) {
    ((LPUNKNOWN)*ppv)->AddRef(); // Add reference if we give a pointer
    return NOERROR;
    }
  else {
  	return ResultFromScode(E_NOINTERFACE);
  	}
  }


STDMETHODIMP_(ULONG) CheckerClassFactory::AddRef() {
  return ++m_cRef;
  }


STDMETHODIMP_(ULONG) CheckerClassFactory::Release() {
  ULONG		UnreleaseRef;

  UnreleaseRef=--m_cRef;

  if (m_cRef == 0)
  	delete this; // No reference left, so delete the CheckerClassFactory

  return UnreleaseRef;
  }

/*
 * CheckerClassFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

STDMETHODIMP CheckerClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
  CheckerShader*      pObj = NULL;
  HRESULT       hr;

  *ppvObj = NULL;
  hr = ResultFromScode(E_OUTOFMEMORY);

  //Verify that a controlling unknown asks for IUnknown
  if (pUnkOuter && !IsEqualIID(riid, IID_IUnknown))
  	return ResultFromScode(E_NOINTERFACE);

  //Create the object passing function to notify on destruction.
  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExShader))
  	pObj = new CheckerShader;
  else
  	return ResultFromScode(E_NOINTERFACE);

  hr=pObj->QueryInterface(IID_I3DExShader, ppvObj);

  //Kill the object if initial creation failed.
  if (FAILED(hr))
  	delete pObj;
  else
  	global_count_Obj++;

  return hr;
  }


/*
 * CheckerClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow in "Shadrdll.cpp".
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

STDMETHODIMP CheckerClassFactory::LockServer(BOOL fLock) {
  if (fLock)
      global_count_Lock++;
  else
      global_count_Lock--;

  return NOERROR;
  }


  
// ****** Class Factory *****

//------------------------------------------------------------------------

RainbowClassFactory::RainbowClassFactory() {
  m_cRef=0L;
  return;
  }


RainbowClassFactory::~RainbowClassFactory() {
  return;
  }

HRESULT RainbowClassFactory::QueryInterface(REFIID riid, LPVOID FAR* ppv) {
  *ppv=NULL;

  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
  	*ppv=(LPVOID)this;

  if (*ppv!=NULL) {
    ((LPUNKNOWN)*ppv)->AddRef();
    return NOERROR;
    }
  else {
  	return ResultFromScode(E_NOINTERFACE);
  	}
  }


ULONG RainbowClassFactory::AddRef() {
  return ++m_cRef;
  }


ULONG RainbowClassFactory::Release() {
  ULONG		UnreleaseRef;

  UnreleaseRef=--m_cRef;

  if (m_cRef == 0) 
   	delete this;

  return UnreleaseRef;
  }

/*
 * ShaderClassFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

HRESULT RainbowClassFactory::CreateInstance(LPUNKNOWN pUnkOuter, REFIID riid, LPVOID FAR* ppvObj) {
  RainbowShader*      pObj = NULL;
  HRESULT       hr;

  *ppvObj = NULL;
  hr = ResultFromScode(E_OUTOFMEMORY);

  //Verify that a controlling unknown asks for IUnknown
  if (pUnkOuter && !IsEqualIID(riid, IID_IUnknown))
  	return ResultFromScode(E_NOINTERFACE);

  //Create the object passing function to notify on destruction.
  if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExShader))
  	pObj = new RainbowShader;
  else
  	return ResultFromScode(E_NOINTERFACE);

  hr=pObj->QueryInterface(IID_I3DExShader, ppvObj);

  //Kill the object if initial creation failed.
  if (FAILED(hr))
  	delete pObj;
  else
  	global_count_Obj++;

  return hr;
  }


/*
 * ShaderClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow.
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

HRESULT RainbowClassFactory::LockServer(BOOL fLock) {
  if (fLock)
      global_count_Lock++;
  else
      global_count_Lock--;

  return NOERROR;
  }


